package com.school.project.service.teacher;

import com.school.project.dto.teacher.AddAssessmentDTO;
import com.school.project.dto.teacher.AddSectionDTO;
import com.school.project.dto.teacher.UploadMarksDTO;
import com.school.project.mapper.MarksToListOfMarks;
import com.school.project.mapper.SectionToListOfSectionMapper;
import com.school.project.model.marks.Assessment;
import com.school.project.model.marks.Section;
import com.school.project.model.marks.StudentMarks;
import com.school.project.repository.CourseRepository;
import com.school.project.repository.StudentRepository;
import com.school.project.repository.marks.AssessmentRepository;
import com.school.project.repository.marks.AssessmentTypeRepository;
import com.school.project.repository.marks.SectionRepository;
import com.school.project.repository.marks.StudentMarksRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;

@Service
public class UploadMarksServiceImpl implements UploadMarksService {

    @Autowired
    CourseRepository courseRepository;
    @Autowired
    AssessmentTypeRepository assessmentTypeRepository;

    @Autowired
    AssessmentRepository assessmentRepository;

    @Autowired
    SectionRepository sectionRepository;

    @Autowired
    StudentRepository studentRepository;

    @Autowired
    SectionToListOfSectionMapper sectionMapper;

    @Autowired
    MarksToListOfMarks marksMapper;

    @Autowired
    StudentMarksRepository studentMarksRepository;
    @Override
    @Transactional
    public Assessment setAssessment(AddAssessmentDTO assessmentDTO) {
        Assessment assessment = new Assessment();

        assessment.setAssessmentType(assessmentTypeRepository.findByAssessmentTypeName(
                assessmentDTO.getAssessment()));

        assessment.setCourse(courseRepository.findByCourseCode(
                assessmentDTO.getCourseCode()));

        return assessmentRepository.save(assessment);
    }

    @Override
    @Transactional
    @Modifying
    public Section setSection(AddSectionDTO sectionDTO) {
        Section section = new Section();

        section.setAssessment(assessmentRepository.findByAssessmentId(sectionDTO.getAssessmentId()));
        section.setTotalMarks(sectionDTO.getTotalMarks());
        section.setTotalWeightage(sectionDTO.getTotalWeightage());

        section = sectionRepository.save(section);

        Assessment assessment = assessmentRepository.findByAssessmentId(sectionDTO.getAssessmentId());
        assessment.setSection(sectionMapper.modelToList(section));

        return section;
    }

    @Override
    @Transactional
    public void setMarks(UploadMarksDTO marksDTO) {

        int marksSize = marksDTO.getObtainedMarks().size();
        Section section = sectionRepository.findById(marksDTO.getSectionId()).get();

/*        List<Student> studentsSet = new ArrayList<>();
        List<Long> studentObtainedMarks = marksDTO.getObtainedMarks();
        studentRollNumbers.forEach(studentRollNumber -> studentsSet.add(studentRepository.findByRollNumber(studentRollNumber)));

        studentsSet.forEach(student -> {
                StudentMarks studentMarks = new StudentMarks();

                studentMarks.setStudent(student);
                studentMarks.setSection(section);
                studentMarks =studentMarksRepository.save(studentMarks);
                student.setStudentMarks(studentMarks);
        });*/git

        for (int i = 0; i<marksDTO.getStudentRollNumbers().size() ;i++) {
            StudentMarks studentMarks = new StudentMarks();

            studentMarks.setSection(section);
            studentMarks.setStudent(studentRepository.findByRollNumber(marksDTO.getStudentRollNumbers().get(i)));

            if( i < marksDTO.getObtainedMarks().size() ) {
                studentMarks.setObtainedMarks(marksDTO.getObtainedMarks().get(i));
            } else {
                studentMarks.setObtainedMarks(0l);
            }

            studentMarksRepository.save(studentMarks);
        }

    }


}
